#!/usr/bin/env ts-node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const simple_git_1 = __importDefault(require("simple-git"));
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const inquirer_1 = __importDefault(require("inquirer"));
const promises_1 = __importDefault(require("fs/promises"));
const child_process_1 = require("child_process");
const program = new commander_1.Command();
program.version('1.0.0');
program
    .option('-d, --dir <dir>', '项目目录，默认当前文件夹', process.cwd())
    .option('-n, --name <name>', '模板名称，对应 Gitee group 下的项目名称', '')
    .option('-b, --branch <branch>', '分支名称，默认 main 分支', 'main');
program.parse(process.argv);
const git = (0, simple_git_1.default)();
// 输入项目目录
function selectProjectDir() {
    return __awaiter(this, void 0, void 0, function* () {
        let projectDir;
        // 使用一个循环来持续请求用户输入，直到得到有效的目录
        while (!projectDir) {
            const answers = yield inquirer_1.default.prompt([
                {
                    type: 'input',
                    name: 'projectDir',
                    message: '请输入项目目录路径：',
                    // 提供一个默认值，例如当前工作目录
                    default: process.cwd(),
                    // 验证用户输入
                    validate: (input) => {
                        const resolvedPath = path.resolve(input);
                        if (!fs.existsSync(resolvedPath) || !fs.statSync(resolvedPath).isDirectory()) {
                            return '目录不存在或不是一个有效的目录，请输入一个有效的目录路径。';
                        }
                        // 如果目录存在且是有效的，返回 true 以结束验证
                        return true;
                    },
                },
            ]);
            // 如果验证通过，则赋值给 projectDir
            if (answers.projectDir) {
                projectDir = answers.projectDir;
            }
        }
        // 返回有效的项目目录路径
        return projectDir;
    });
}
//选择模版类型==》vue、react
function selectTemplate() {
    return __awaiter(this, void 0, void 0, function* () {
        // 读取 config/templates.json 文件
        const templatesFilePath = path.join(__dirname, '..', 'config', 'templates.json');
        try {
            const templatesData = yield promises_1.default.readFile(templatesFilePath, 'utf8');
            const { templates } = JSON.parse(templatesData);
            // 使用 inquirer 提示用户选择模板
            const { template } = yield inquirer_1.default.prompt([
                {
                    type: 'list',
                    name: 'template',
                    message: '请选择一个模板:',
                    choices: templates,
                },
            ]);
            return template;
        }
        catch (err) {
            console.error('读取模板配置文件时出错:', err);
            throw err;
        }
    });
}
// async function selectTemplate() {
//     const templates = [
//       { name: 'Vue', value: 'http://code.devops.piccnet/picc/picc__picc-wechat-sale-platform-etong/enterprise-insurance-management.git' },
//       { name: 'React', value: 'http://code.devops.piccnet/picc/picc__picc-wechat-sale-platform-etong/downloadapks_pad.git' },
//       // ...更多模板
//     ];
//     const { template } = await inquirer.prompt([
//       {
//         type: 'list',
//         name: 'template',
//         message: '请选择一个模板:',
//         choices: templates,
//       },
//     ]);
//     return template;
//   } 
//选择分支
function selectBranch() {
    return __awaiter(this, void 0, void 0, function* () {
        // 读取 config/branches.json 文件
        const branchesFilePath = path.join(__dirname, '..', 'config', 'branchs.json');
        try {
            const branchesData = yield promises_1.default.readFile(branchesFilePath, 'utf8');
            const { branches } = JSON.parse(branchesData);
            // 使用 inquirer 提示用户选择分支
            const { branch } = yield inquirer_1.default.prompt([
                {
                    type: 'list',
                    name: 'branch',
                    message: '请选择一个分支:',
                    choices: branches,
                },
            ]);
            return branch;
        }
        catch (err) {
            console.error('读取分支配置文件时出错:', err);
            throw err;
        }
    });
}
// async function selectBranch() {
//     const branchList = [
//       { name: 'main', value: 'main' },
//       { name: 'starter-template', value: 'starter-template' },
//       // ...更多分支
//     ];
//     const { branch } = await inquirer.prompt([
//       {
//         type: 'list',
//         name: 'branch',
//         message: '请选择一个分支:',
//         choices: branchList,
//       },
//     ]);
//     return branch;
// }
// 克隆模版到指定项目目录
function cloneRepo() {
    return __awaiter(this, void 0, void 0, function* () {
        // 让用户输入项目目录路径
        const projectDir = yield selectProjectDir();
        // 获取用户选择的模板
        const repoUrl = yield selectTemplate();
        // 获取用户选择的分支
        const branch = yield selectBranch();
        console.log(`正在克隆仓库 ${repoUrl} 到 ${projectDir} 分支 ${branch}`);
        // git.clone(repoUrl, projectDir, { '--branch': branch })
        //     .then(() => {
        //         console.log(`项目已成功克隆到 ${projectDir}`);
        //     })
        //     .catch((err) => {
        //         console.error(`克隆项目时出错: ${err.message}`);
        //     });
        // 执行git clone命令
        const cloneProcess = (0, child_process_1.spawn)('git', ['clone', '-b', branch, '--progress', repoUrl, projectDir]);
        // 监听标准输出以显示克隆进度
        cloneProcess.stdout.on('data', (data) => {
            process.stdout.write(data.toString());
        });
        // 监听标准错误以处理错误情况
        cloneProcess.stderr.on('data', (data) => {
            process.stderr.write(data.toString());
        });
        // 监听克隆命令的退出事件
        cloneProcess.on('close', (code) => {
            if (code === 0) {
                console.log(`项目已成功克隆到 ${projectDir}`);
            }
            else {
                console.error('克隆失败,请重试');
            }
        });
    });
}
cloneRepo();
